@@ -44,6 +44,7 @@ module Lens.Micro.TH
   lensRulesFor,
   classyRules,
   camelCaseFields,
+  abbreviatedFields,
 
   -- * Configuring lens rules
   lensField,
@@ -61,7 +62,6 @@ import           Control.Monad
 import           Data.Char
 import           Data.Data
 import           Data.Either
-import           Data.Foldable (toList)
 import qualified Data.Map as Map
 import           Data.Map (Map)
 import           Data.Monoid
@@ -376,7 +376,7 @@ instance HasY (Foo a) a where
 
 If you only want to make lenses for some fields, you can prefix them with underscores – the rest would be untouched. If no fields are prefixed with underscores, lenses would be created for all fields.
 
-The prefix must be the same as the name of the name of the data type (/not/ the constructor).
+The prefix must be the same as the name of the name of the data type (/not/ the constructor). If you don't like this behavior, use @'makeLensesWith' 'abbreviatedFields'@ – it allows any prefix (and even different prefixes).
 
 If you want to use 'makeFields' on types declared in different modules, you can do it, but then you would have to export the @Has*@ classes from one of the modules – 'makeFields' creates a class if it's not in scope yet, so the class must be in scope or else there would be duplicate classes and you would get an “Ambiguous occurrence” error.
 
@@ -638,6 +638,36 @@ camelCaseNamer tyName fields field = maybeToList $ do
   computeMethod (x:xs) | isUpper x = Just (toLower x : xs)
   computeMethod _                  = Nothing
 
+{- |
+Like standard rules used by 'makeFields', but doesn't put any restrictions on the prefix. I.e. if you have fields called
+
+* @_fooBarBaz@
+* @_someX@
+* @someY@
+
+then the generated lenses would be called @barBaz@ and @x@.
+-}
+abbreviatedFields :: LensRules
+abbreviatedFields = defaultFieldRules { _fieldToDef = abbreviatedNamer }
+
+abbreviatedNamer :: Name -> [Name] -> Name -> [DefName]
+abbreviatedNamer _ fields field = maybeToList $ do
+
+  fieldPart <- stripMaxLc (nameBase field)
+  method    <- computeMethod fieldPart
+  let cls = "Has" ++ fieldPart
+  return (MethodName (mkName cls) (mkName method))
+
+  where
+  stripMaxLc f = do x <- stripPrefix optUnderscore f
+                    case break isUpper x of
+                      (p,s) | null p || null s -> Nothing
+                            | otherwise        -> Just s
+  optUnderscore  = ['_' | any (isPrefixOf "_" . nameBase) fields ]
+
+  computeMethod (x:xs) | isUpper x = Just (toLower x : xs)
+  computeMethod _                  = Nothing
+
 defaultFieldRules :: LensRules
 defaultFieldRules = LensRules
   { _simpleLenses    = True
@@ -1262,9 +1292,7 @@ applyTypeSubst sub = rewrite aux
 ------------------------------------------------------------------------
 
 {- |
-Rules used to generate lenses. You can't create them from scratch, but you can customise already existing ones with lenses in the “Configuring lens rules” section.
-
-For an example, see 'makeLensesWith'.
+Rules used to generate lenses. The fields are intentionally not exported; to create your own rules, see lenses in the “Configuring lens rules” section. You'd have to customise one of the existing rulesets; for an example of doing that, see 'makeLensesWith'.
 -}
 data LensRules = LensRules
   { _simpleLenses    :: Bool
@@ -1295,16 +1323,17 @@ data DefName
 -- Template Haskell wants type variables declared in a forall, so
 -- we find all free type variables in a given type and declare them.
 quantifyType :: Cxt -> Type -> Type
-quantifyType c t = ForallT vs c t
-  where
-  vs = map PlainTV (toList (setOf typeVars t))
+quantifyType = quantifyType' Set.empty
 
 -- This function works like 'quantifyType' except that it takes
 -- a list of variables to exclude from quantification.
 quantifyType' :: Set Name -> Cxt -> Type -> Type
 quantifyType' exclude c t = ForallT vs c t
   where
-  vs = map PlainTV (toList (setOf typeVars t Set.\\ exclude))
+    vs = map PlainTV
+       $ filter (`Set.notMember` exclude)
+       $ nub -- stable order
+       $ toListOf typeVars t
 
 
 ------------------------------------------------------------------------
